/*
 * Project name:
   Bluetooth System control
     
 * Copyright:
   Alexander Wolf

  * Revision History:
   20131125:

 * Description:
   Small Example of BT2 Click on PIC32 & Fusion V7 Board.
   Establish a SPP connection and comunication via terminal UART.
   Turn on/off LED on the board via BT

 * Test configuration:
   MCU:             PIC32MX
   dev.board:       Fusion V7
   Bluetooth Click 2   BlueGiga WT41

 * NOTES:
  - Put Bluetooth Click2 into mikroBus socket 2 !!! not socket 1 !! UART B is used for BT2 Click
  - Install a terminal program on your Computer or smart phone (like BluetoothSPP for Android)
  - Connect the Fusion board UART-USB UART A to Computer and connect it to a Terminal Program
  - On Windows turn Bluetooth ON. Search for new devices and connect to BT2 Stick. Windows will
    install a new Com Port linked to BT2Click. When you Start a Terminal and connect to this Comport.
    Now you can send messages between PIC32 fusion Board and Computer.
  - Swich on LED on Port D
  - for LED on send L1, for LED off send L0, Send CR after message

    BlueGiga WT41
    Find complete list of commands here:
    http://www.bluegiga.com/files/bluegiga/Presentations/iWRAP4_presentation.pdf
*/

#define BUFFER_SIZE 10

// List of receied comands
const char _SET_LIGHT_ON[]   = {'L','1',13,0}; //  0 == important for strstr
const char _SET_LIGHT_OFF[]  = {'L','0',13,0};


// List of commands
const char _BT_SYSTEM_ON[]   = "AT";                 // software reset
const char _BT_SET_RESET[]   = "SET RESET";          // returns the factory settings of the module
const char _BT_RESET[]       = "RESET";              // software reset
const char _BT_INFO[]        = "INFO";               // software information
const char _BT_SET_BT_PAIR[] = "SET BT PAIR * ";     // used for pairings
const char _BT_SET_BT_AUTH[] = "SET BT AUTH * 1234";     // used to set Authentication pin
const char _BT_LIST[]        = "LIST";               // list current established connections
const char _BT_INQUIRY[]     = "INQUIRY 7";          // do inquiry of currently visible BT devices
const char _BT_SET_BT_NAME[] = "SET BT NAME ";       // set name of the bluetooth device
const char _BT_CALL[]        = "CALL XX:XX:XX:XX:XX:XX 1 RFCOMM";  // CALL function example, instead of XX put remote BT device MAC


char txt[BUFFER_SIZE];
unsigned short i=0, DataReady =0;
char uart_rd;


// - UART 2 Terminal
void UART2interrupt() iv IVT_UART_2 ilevel 6 ics ICS_AUTO {
  uart_rd = UART2_Read();        // read the received data
  UART5_Write(uart_rd);          // and send data via UART
  U2RXIF_bit = 0;                // Interrupt register flag zurcksetzen auf 0
}

void UART5interrupt() iv IVT_UART_5 ilevel 6 ics ICS_AUTO {
  uart_rd = UART5_Read();        // read the received data
  UART2_Write(uart_rd);          // and send data via UART
  U5RXIF_bit = 0;                // Interrupt register flag zurcksetzen auf 0

  /**/
  if (uart_rd == 13) {          // check for CR
    txt[i] = 13;
    txt[i+1] = 0;              // Puting 0 at the end of the string
    DataReady = 1;           // Data is received
  }
  else {
  //    if (uart_rd != 10)
    {
      txt[i] = uart_rd;                          // Moving the data received from UART to string txt[]
      i++;
    }
    }
}



// Wait
void Wait(){
  Delay_ms(1000);
}

void main() {

  AD1PCFG = 0xFFFF;             // Configure AN pins as digital I/O
  JTAGEN_BIT = 0;
  
  TRISD = 0;             // Initialize PORTA as output
  LATD = 0;              // Set PORTA to zero
  
  U2IP0_bit = 0;                     // Set UART2 interrupt
  U2IP1_bit = 1;                     // Set interrupt priorities
  U2IP2_bit = 1;                     // Set UART2 interrupt to level 6

  U5IP0_bit = 0;                     // Set UART2 interrupt
  U5IP1_bit = 1;                     // Set interrupt priorities
  U5IP2_bit = 1;                     // Set UART2 interrupt to level 6

  UART2_Init(115200);            // Terminal
  Delay_ms(100);
  UART5_Init(115200);           // Bluetooth
  Delay_ms(100);
  

  
  U2RXIE_bit = 1;                    // Set UART Receive Interrupt
  U5RXIE_bit = 1;
  EnableInterrupts();                // Enable interruts as previously set
  
/// START Programm ///////////////////////////

  Uart2_write_text("uController started");
  Uart2_write(13);

  Uart5_write_text(_BT_SET_RESET);         // The SET RESET command returns the factory settings of the module
  Uart5_write(13);                    // CR
  Wait();
  Uart2_write_text("Reset BT");
  Uart2_write(13);
 
  Uart5_write_text(_BT_SYSTEM_ON);         // The AT command returns OK
  Uart5_write(13);                   // CR
  Wait();

  Uart5_write_text(_BT_SET_BT_AUTH);         //
  Uart5_write(13);                    // CR
  Wait();
  Uart2_write_text("Code: 1234");
  Uart2_write(13);

  Uart5_write_text(_BT_INFO);         // The AT command returns OK
  Uart5_write(13);                  // CR
  Wait();

  Uart5_write_text(_BT_SET_BT_PAIR);       // If command SET BT PAIR * is given, all pairings will be removed
  Uart5_write(13);                   // CR
  Wait();
  Wait();
  Uart2_write_text("Bluetooth online");
  Uart2_write(13);

  i = 0;
  DataReady = 0;
  
  while (1) {
    WDTCONSET = 0x01;                // Watchdog service
   if (DataReady){
      if (strstr(txt, _SET_LIGHT_ON)){
        LATD = 0xFFFF;              // Set PORTA to zero
        Uart2_write_text("Light an");
        Uart2_write(13);

      }
      if (strstr(txt, _SET_LIGHT_OFF)){
        LATD = 0X0000;              // Set PORTA to zero
        Uart2_write_text("Light aus");
        Uart2_write(13);
      }

      DataReady = 0;
      i = 0;                         // reset i counter
      memset(txt, 0, BUFFER_SIZE);   // reset message content

    }/*  */
  }
}