/*
 * Project name:
     Bluetooth2 click;
 * Copyright:
     (c) MikroElektronika, 2005-2011.
  * Revision History:
     20130205:
       - initial release (JK);
 * Description:
     This is a simple project which demonstrates the use of the BlueTooth2 click board.
     Bluetooth2 click board should be placed on development system and it will Wait for master to establish connection.
     After connection is set user can send text messages from Serial Terminal and text will be displayed on LCD.
     Text messages should end with CR + LF.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412D.pdf
     dev.board:       EasyPIC v7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL 32.0000 MHz, 8.0000 MHz Crystal
     Ext. Modules:    BlueTooth2 click : ac:Bluetooth2_click
                      http://www.mikroe.com/click/bluetooth2
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
  - Place Bluetooth2 stick board into mikroBUS socket1.
  - Note that BlueGiga WT41 has default baudrate 115200bps and that you need to change the
    its firmware in order to run this example at 19200bps.
    Find complete list of commands here:
    http://www.bluegiga.com/files/bluegiga/Presentations/iWRAP4_presentation.pdf
  - Turn ON the Lcd Backlight (SW4.6)
*/

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

// List of commands
const char _BT_SET_RESET[]   = "SET RESET";          // returns the factory settings of the module
const char _BT_RESET[]       = "RESET";              // software reset
const char _BT_SET_BT_PAIR[] = "SET BT PAIR * ";     // used for pairings
const char _BT_SET_BT_AUTH[] = "SET BT AUTH * ";     // used to set Authentication pin
const char _BT_LIST[]        = "LIST";               // list current established connections
const char _BT_INQUIRY[]     = "INQUIRY 7";          // do inquiry of currently visible BT devices
const char _BT_SET_BT_NAME[] = "SET BT NAME ";       // set name of the bluetooth device
const char _BT_CALL[]        = "CALL XX:XX:XX:XX:XX:XX 1 RFCOMM";  // CALL function example, instead of XX put remote BT device MAC

char BT_Ring_string[]        = "RING";
char BT_No_Carrier_String[]  = "NO CARRIER";

char txt[100];
unsigned short i, tmp, DataReady, DataOut;

// Uart Rx interrupt handler
void interrupt(){
  if (PIR1.RCIF == 1) {                        // Do we have uart rx interrupt request?
    tmp = UART1_Read();                        // Get received byte
      if (tmp == 13) {                         // check for CR
        txt[i] = 0;                            // Puting 0 at the end of the string
        DataReady = 1;                         // Data is received
      }
      else {
        if (tmp != 10){
          txt[i] = tmp;                          // Moving the data received from UART to string txt[]
          i++;
        }
      }
   PIR1.RCIF = 0;
   }
}

// Sends constant message via UART (without carriage return)
void BT_Send(const char *text){
char Str[30];
char *pStr;
  pStr = &Str;
  while (*pStr++ = *text++)
    ;
  UART_Write_Text(Str);
}

// Sends message via UART (without carriage return)
void BT_Send_RAM(char *text){
  UART_Write_Text(text);
}

// Send Cariage return via UART to BlueTooth module
void BT_Send_CR(){
  UART_Write(0x0D);
}

// Wait
void Wait(){
  Delay_ms(1000);
}

void main() {
  ANSELB = 0;
  ANSELC  = 0;                    // Configure AN pins as digital I/O
  SLRCON = 0;
  
  UART1_Init(115200);             // Initialize UART1 module
  Wait();

  LCD_Init();                     // LCD Init
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);

  Lcd_Out(1,1,"Configuring");
  Lcd_Out(2,1,"BT module");
  
  BT_Send(_BT_SET_RESET);         // The SET RESET command returns the factory settings of the module
  BT_Send_CR();                   // CR
  Wait();
  
  BT_Send(_BT_SET_BT_PAIR);       // If command SET BT PAIR * is given, all pairings will be removed
  BT_Send_CR();                   // CR
  Wait();
  
  BT_Send(_BT_RESET);             // Command RESET is used to perform a software reset
  BT_Send_CR();                   // CR
  Wait();
 
  BT_Send(_BT_SET_BT_AUTH);       // Set PIN Code Authentication
  BT_SEND("1234");                // Set desired PIN CODE
  BT_Send_CR();                   // CR
  
  BT_Send(_BT_SET_BT_NAME);       // Set desired name of the bluetooth module
  BT_Send("Bluetooth2 click");
  BT_Send_CR();                   // CR

  Wait();
  Wait();
  
  Lcd_Out(1,1,"Waiting for");
  Lcd_Out(2,1,"connection");
  
  DataReady = 0;                              // reset data ready flag
  // setup interrupts
  PIE1.RCIE   = 1;                            // enable RX interrupt
  INTCON.PEIE = 1;                            // enable peripheral interrupts
  INTCON.GIE  = 1;                            // Interrupts allowed
  Wait();
  
  while (1) {
    // if we have received message
    if (DataReady){
      INTCON.GIE  = 0;                        // Disable interrupts
      DataReady = 0;                          // reset data ready flag
      DataOut = 1;                            // Set data output flag
      if (strstr(txt, BT_Ring_string)){       // If the message was RING
        LCD_Cmd(_LCD_CLEAR);
        Lcd_Out(1,1,"Connected!");            // Displat Connected on LCD
        Wait();
        LCD_Cmd(_LCD_CLEAR);                  // Clear display
        Lcd_Out(1,1,"Receiving...");
        DataOut = 0;                          // reset data output flag
      }
      
      if (strstr(txt, BT_No_Carrier_String)){ // If the message was No Carrier (disconnect)
        LCD_Cmd(_LCD_CLEAR);
        Lcd_Out(1,1,"Disconnected!");         // Display disconnect on LCD
        DataOut = 0;                          // reset data output flag
      }
      
      if (DataOut){                           // if data output flag (we can display data on screen)
        LCD_Cmd(_LCD_CLEAR);                  // Clear display
        Lcd_Out(1,1,"Received:");
        Lcd_Cmd(_LCD_SECOND_ROW);
        i = 0;
        while (txt[i] != 0){
          Lcd_Chr_CP(txt[i]);                 // Displaying the received text on the LCD
          i++;
        }
      }
      i = 0;                                  // reset i counter
      memset(txt, 0, 100);                    // reset message content
      INTCON.GIE  = 1;                        // Enable interrupts
    }
  }
}